#include <stm32f401xc.h>
#include <stdint.h>
#include "delay.h"
#include "nokia_glcd.h"
#include "gpio.h"
#include "ctype.h"

  const uint8_t* font; 
  uint8_t xSize;
  uint8_t ySize;
  uint8_t ySize8;
  uint8_t firstCh;
  uint8_t lastCh;
  uint8_t minCharWd;
  uint8_t minDigitWd;
  uint8_t cr;  // carriage return mode for printStr
  uint8_t dualChar;
  uint8_t invertCh,invertMask;
	bool (*isNumberFun)(uint8_t ch);
//----max function-------------------------
int max(int a, int b) {

return (a > b) ? a : b;

}

//--------------------------------------

bool N5110_isNumber(uint8_t ch)
{
  return isdigit(ch) || ch==' ';
}
//--------------------------------
char pgm_read_byte(char *p){
  return *p;
}
//---------------------------------
void setDat() {gpio_write(DC, 1); }
void setCmd() { gpio_write(DC, 0); }
void startCS() { gpio_write(CS, 0); }
void stopCS() { gpio_write(CS, 1); }

//uint8_t x,y;
/* The function sends a byte of data through SPI */
/* argument d: the byte to be sent */
/* return value: the received data */
uint8_t spi2_transfer(uint8_t d)
{
	
	gpio_write(CS,0);
	SPI2->DR = d;	/* send the contents of d */
	while((SPI2->SR&(1<<0)) == 0);	/* wait until RXNE is set */
	
	
 gpio_write(CS,1);	
	return SPI2->DR;	/* return the received data */ 
}

void glcd_cmd(uint8_t cmd)
{

	gpio_write(DC,0);
	spi2_transfer(cmd);
}

void glcd_data(uint8_t data)
{
	
	gpio_write(DC,1);
	spi2_transfer(data);
}

void glcd_init(void)
{
//--------------------------------------
	  //isNumberFun = &isNumber;
  cr = 0;
  font = NULL;
  dualChar = 0;
	//--------------------------------------
RCC->APB1ENR |=RCC_APB1ENR_SPI2EN;	
gpio_output(CS);
gpio_output(DC);
gpio_output(RESET);	
gpio_init(MOSI, GPIO_MODE_AF, GPIO_OTYPE_PUSH_PULL, GPIO_SPEED_INSANE,GPIO_PULL_NONE, 5);
gpio_init(SLK, GPIO_MODE_AF, GPIO_OTYPE_PUSH_PULL, GPIO_SPEED_INSANE,GPIO_PULL_NONE, 5);

	SPI2->CR1 = 0x35C;	/* SPE = 1, BR = 3, FFD = 0, SSI and SSM = 1 */	
	//GPIOB->BSRR = (1<<GLCD_CS)|(1<<GLCD_RESET);	/* make CS and RESET pins high */
	
	SPI2->CR1 = 0x35C;	/* SPE = 1, BR = 3, FFD = 0, SSI and SSM = 1 */	
	gpio_write(DC, 1);
	gpio_write(RESET, 1);	/* make CS and RESET pins high */
	
	Delay_ms(10);
	gpio_write(RESET, 0);	/* reset the LCD (RESET = 0) */
	Delay_ms(70);
	gpio_write(RESET, 1); /* release the RESET pin */
	
	glcd_cmd(0x21);	/* switch to extended command mode */
	glcd_cmd(0x06);	/* set temp. coefficient 2 */
	glcd_cmd(0x13);	/* set LCD bias mode 1:48 */
	glcd_cmd(0xC2); /* set VOP to 7V (VOP = 66) */
	glcd_cmd(0x20); /* switch to basic mode */
	glcd_cmd(0x0C);	/* set lcd display to normal mode */

}

void gotoXY(uint8_t x, uint8_t y) 
{
	
	glcd_cmd(0x80 | x);	/* set x */
	glcd_cmd(0x40 | y);	/* set y (bank) */
}


void glcd_setcontrast(uint8_t contrast)
{
	/* Max Contrast */
	if(contrast > 0x7F) contrast = 0x7F;
	
	glcd_cmd(0x21);
	glcd_cmd(0x80 | contrast);
	glcd_cmd(0x20);

}
void glcd_clear(void) 
{
	uint16_t i;

	gotoXY(0, 0);
	
	for (i = 0 ; i < 504 ; i++)
		glcd_data(0x00);		
}
//------------------------------------------------------------------
void N5110_setFont(const uint8_t* f)
{
  font     = f;
  xSize    =-(font[0]+0);
  ySize    = (font[0]+1);
  firstCh  = (font[0]+2);
  lastCh   = (font[0]+3);
  ySize8   = (ySize+7)/8;
  minCharWd  = 0;
  minDigitWd = 0;
  cr = 0;
  invertCh = 0;
  invertMask = 0xff;
}

int N5110_printChar(int x, uint8_t row, uint8_t _ch)
{

  int ch = _ch;

  
  if(!font || ch<firstCh || ch>lastCh || x>=SCR_WD || row>=SCR_HT/8) return 0;

  int j,i, idx = 4 + (ch - firstCh)*(xSize*ySize8+1);
int  wd = (font[0] + idx++);
  int wdL = 0, wdR = 1; // default spacing before and behind char

  if((*isNumberFun)(ch)) {
    if(minDigitWd>wd) {
      wdL = (minDigitWd-wd)/2;
      wdR += (minDigitWd-wd-wdL);
    }
  } else
  if(minCharWd>wd) {
    wdL = (minCharWd-wd)/2;
    wdR += (minCharWd-wd-wdL);
  }
	
	
  if(x+wd+wdL+wdR>SCR_WD) wdR = max(SCR_WD-x-wdL-wd, 0);
  if(x+wd+wdL+wdR>SCR_WD) wd = max(SCR_WD-x-wdL, 0);
  if(x+wd+wdL+wdR>SCR_WD) wdL = max(SCR_WD-x, 0);
  for(j=0; j<ySize8; j++) {
    gotoXY(x, row+j);
    setDat();
	  startCS();
    if(!invertCh) {
      for(i=0; i<wdL; i++) glcd_data(0);
      for(i=0; i<wd; i++)  glcd_data((font[0]+idx+i*ySize8+j));
      for(i=0; i<wdR; i++) glcd_data(0);
    } else {
      for(i=0; i<wdL; i++) glcd_data(invertMask);
      for(i=0; i<wd; i++)  glcd_data((font[0]+idx+i*ySize8+j)^invertMask);
      for(i=0; i<wdR; i++) glcd_data(invertMask);
			
    }
    stopCS();
		
  }
	
  return wd+wdL+wdR;
}
